package org.objectweb.telosys.common;

import java.io.InputStream;
import java.util.Properties;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.xml.parsers.DocumentBuilderFactory;

import org.objectweb.telosys.rpl.xml.IXmlWrapperProvider;

/**
 * General Telosys environment informations and utilities
 * 
 * @author Laurent GUERIN
 *  
 */
public final class Telosys
{
	private static final String   SAX_XERCES  = "org.apache.xerces.parsers.SAXParser";
	
	private static final String   SAX_CRIMSON = "org.apache.crimson.parser.XMLReaderImpl";

    private static final TelosysClassLogger $log                = new TelosysClassLogger(Telosys.class);

    //-----------------------------------------------------------------------------
    // PROPERTIES & CONFIGURATION
    //-----------------------------------------------------------------------------
    //--- Telosys config file ( absolute path )
    //private static String                   $TelosysPropFile   = null;
    
    private static TelosysProperties        $TelosysProperties = null;
    
    //--- The properties loaded from "telosys.properties"
    private static Properties               $properties = null;

    //--- The properties loaded from "telosys.properties"
    private static int                      $iTelosysPropertiesOrigin = TelosysProperties.UNKNOWN ;

    //--- Instance of the "Framework Internal Factory"
    private static TelosysFactory           $telosysFactory    = null;

    //--- Trace configuration file 
    //private static String                   $sTraceConfFile    = null;

    //--- Authentication required to create ScreenSession 
    private static boolean                  $bAuthenticationRequired = true ;

    //--- XML SAX Parser Class Name ( for SAX 2 XMLReaderFactory.createXMLReader() )
    private static String                   $sSAXParserClassName     = null;
    
    private static LogWriter                $TelosysLogger     = null;

    //-----------------------------------------------------------------------------
    // WEB APP 
    //-----------------------------------------------------------------------------
    //--- Telosys Web App root directory
    private static String                   $TelosysWebAppRootDir = null;

    //--- Telosys Web App Context Name ( from web.xml : tag "display-name" )
    private static String                   $TelosysWebAppName = null;
    
    //--- Telosys config directory (directory where "telosys.properties" was found)
    //private static String                   $TelosysConfDir    = null;
    
    private static ServletContext           $ServletContext = null;
    //-----------------------------------------------------------------------------
    
    //-----------------------------------------------------------------------------
//    private static void initTraceConf(Properties prop)
//    {
//        if (prop != null)
//        {
//            //--- Nom de domaine pour l'authentification "basic HTTP"
//            $sTraceConfFile = prop.getProperty("TraceConfFile");
//            if ($sTraceConfFile == null)
//            {
//                $log.info("initParameters : No trace config file ('TraceConfFile' not found) ");
//            }
//            else
//            {
//                $sTraceConfFile = $sTraceConfFile.trim() ;
//                $sTraceConfFile = Telosys.transformFileName($sTraceConfFile);
//                if ( Tool.fileExists( $sTraceConfFile ) )
//                {
//                    TraceConfig.init($sTraceConfFile);
//                }
//                else
//                {
//                    Telosys.error("Telosys.initTraceConf() : File " + $sTraceConfFile + " not found !");                                        
//                }        		                    
//            }
//        }
//        else
//        {
//            $log.error("initParameters : properties = null !");
//        }
//    }

    //-----------------------------------------------------------------------------
    private static void createLogWriter(Properties prop)
    {
        if (prop != null)
        {
            $log.info("Create LogWriter with properties. ");
            $TelosysLogger = new LogWriter(prop);
        }
        else
        {
            $log.error("initParameters : properties = null !");
            $log.info("Create LogWriter without properties.  ");
            $TelosysLogger = new LogWriter();
        }
    }

    //-----------------------------------------------------------------------------
    private static boolean checkNotInitialized()
    {
        if (isInitialized())
        {
            //--- Already initialized
            $log.info("INIT : Telosys.init(properties) : WARNING - Already initialized => No change !");
            return false; // Do not init again
        }
        else
        {
            return true; // OK to init
        }
    }

    //-----------------------------------------------------------------------------
    private static void initParamAuthentication(Properties prop)
    {
        //--- Authentication required ?
        $bAuthenticationRequired = true ;                    
        String sProperty = prop.getProperty("AuthenticationRequired");
        //log.info("INIT : Property Authentication Required = " + sAuthenticationRequired );
        if ( sProperty != null )
        {
            String sPropertyTrimLC = sProperty.trim().toLowerCase();
            //log.info("INIT : toLowerCase = '" + sPropTrimLC + "'");
            if ( sPropertyTrimLC.equals("false") )
            {
                //log.info("INIT : Flag is false" );
                $bAuthenticationRequired = false ; 
            }
            else
            {
                //log.info("INIT : Flag is NOT false" );                    
            }
        }        
    }
    //-----------------------------------------------------------------------------
    private static void initParamXmlSaxParser(Properties prop)
    {
        //--- Authentication required ?
        $sSAXParserClassName = null ;                    
        String sProperty = prop.getProperty("SAXParserClassName");
        if ( sProperty != null )
        {
            $sSAXParserClassName = sProperty.trim();
        }        
        else
        {
            $sSAXParserClassName = SAX_CRIMSON ;
        }
    }
    //-----------------------------------------------------------------------------
//    private static void initParamXmlDomParser(Properties prop)
//    {
//        //--- Authentication required ?
//        $sXmlDomParser = null ;                    
//        String sProperty = prop.getProperty("XmlDomParser");
//        if ( sProperty != null )
//        {
//            $sXmlDomParser = sProperty.trim();
//        }        
//    }
    //-----------------------------------------------------------------------------
    private static void initTelosys(IExternalMainFactory extFactory, TelosysProperties tp, ServletContext servletContext)
    {
        if ( tp != null )
        {
            $TelosysProperties = tp ;
            $iTelosysPropertiesOrigin = tp.getOrigin() ;
            Properties prop = tp.getProperties();
            if ( prop != null )
            {
                //--- Global init 
                initTelosys(extFactory, prop);
                
//                //--- Init the Trace Manager with the same origin as the main properties 
//                String s = tp.getTracePropertiesFile();
//                TelosysProperties tpTrace = TelosysPropertiesLoader.loadProperties(s, tp.getOrigin(), servletContext );
//                if ( tpTrace != null )
//                {
//                    TraceConfig.init(tpTrace.getProperties());
//                    $log.info("INIT : Telosys trace configuration initialized. ");
//                }
//                else
//                {
//                    $log.error("initTelosys() : cannot load TraceConfig file : " + s );
//                }
                
                //--- Trace manager init
                initTraceConfig();
                
                //--- Create the log writer
                createLogWriter($properties);
                $log.info("INIT : Telosys log writer created. ");
            }
            else
            {
                $log.error("initTelosys() : cannot get Properties from TelosysProperties !");
            }
        }
        else
        {
            $log.error("initTelosys() : TelosysProperties == null !");
        }
    }
    
    /**
     * Telosys initialization from properties
     * 
     * @param extFactory
     * @param prop
     * @return true if Ok / false if Error
     */
    private static boolean initTelosys(IExternalMainFactory extFactory, Properties prop)
    {
        if (checkNotInitialized())
        {
            //--- Check arguments
            if (extFactory == null)
            {
                $log.error("initTelosys() : No external factory ( extFactory == NULL ) !");
                return false;
            }

            if (prop == null)
            {
                $log.error("initTelosys() : properties = null !");
                return false;
            }

            //--- Build the Telosys factory with the "external factory"
            //$externalMainFactory = extFactory;
            $telosysFactory = new TelosysFactory(extFactory);
            $log.info("INIT : Telosys factory created. ");

            //--- Store the properties
            $properties = prop;
            $log.info("INIT : Telosys properties set. ");

            //--- Authentication required ?
            initParamAuthentication(prop);
            $log.info("INIT : Flag Authentication Required = " + $bAuthenticationRequired );

            //--- XML SAX parser
            initParamXmlSaxParser(prop);
            $log.info("INIT : XML SAX parser class = '" + $sSAXParserClassName + "'" );
            
            $log.info("INIT : Other config files :" );
            $log.info(" . Traces    : '" + getTraceConfigFileName() + "'" );
            $log.info(" . Databases : '" + getDbConfigFileName() + "'" );
            $log.info(" . Screens   : '" + getScreensConfigFileName() + "'" );
            $log.info(" . Tags      : '" + getTagsConfigFileName() + "'" );
            //--- Init trace config & logger
//            initTraceConf($TelosysProperties);
//            createLogWriter($TelosysProperties);
//            $log.info("INIT : Telosys log writer created. ");
        }
        return true;
    }

    private static boolean initTraceConfig()
    {
        boolean bRet = false ;
        Telosys.info("INIT : Init trace properties ( file = '" + Telosys.getTraceConfigFileName() + "' ) ..." );
        InputStream is = Telosys.getTraceConfigInputStream();
        if ( is != null )
        {
            TraceConfig.init(is); 
            Telosys.info("INIT : Trace manager initialized.");                    
            Tool.closeInputStream(is);
        }
        else
        {
            Telosys.error("TelosysUIL.init() : Cannot get TAGS CONFIG InputStream ! ");
        }
        return bRet ;
    }
    
    //-----------------------------------------------------------------------------
//    /**
//     * Telosys initialization from a properties file
//     * 
//     * @param extFactory
//     * @param sPropFile
//     * @return true if Ok / false if Error
//     */
//    private static boolean initTelosys(IExternalMainFactory extFactory, String sPropFile)
//    {
//        $log.info("INIT : Telosys.init(" + sPropFile + ") ... ");
//        Properties propLoaded = Tool.loadProperties(sPropFile);
//        if (propLoaded != null)
//        {
//            //--- Store the file name
//            $TelosysPropFile = sPropFile ;
//            if ( $TelosysPropFile != null )
//            {
//                //--- Store the directory
//                File file = new File($TelosysPropFile); 
//                $TelosysConfDir = file.getParent();
//            }
//            //--- Initialize Telosys with properties loaded
//            return initTelosys(extFactory, propLoaded);
//        }
//        else
//        {
//            $log.error("initTelosys() : cannot load properties from '" + sPropFile + "' ");
//            return false;
//        }
//    }

    //-----------------------------------------------------------------------------
    // PUBLIC METHODS
    //-----------------------------------------------------------------------------
    //-----------------------------------------------------------------------------
    /**
     * Returns the Telosys framework version ( e.g. "1.2.3" )
     * @return
     */
    public static String getVersion()
    {
        return TelosysVersion.VERSION ;
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns true if Telosys is initialized <br>
     * ( true if the properties are loaded and the factory exists )
     * @return
     */
    public static boolean isInitialized()
    {
        if ($properties != null && $telosysFactory != null)
        {
            return true;
        }
        return false;
    }

    //-----------------------------------------------------------------------------
//    /**
//     * Initialize Telosys with a set of properties
//     * 
//     * @param prop
//     *            (properties to use for initialization)
//     */
//    public static void init(IExternalMainFactory extFactory, Properties prop)
//    {
//        initTelosys(extFactory, prop);
//    }

    //-----------------------------------------------------------------------------
    /**
     * Try to initialize the Telosys properties : try to find 'telosys.properties' using the classpath if the file is find, the properties are loaded and Telosys is initialized.
     * 
     * @return the properties
     */
    public static Properties init(IExternalMainFactory extFactory)
    {
        if (checkNotInitialized())
        {
            $log.info("INIT : Telosys.init(extFactory) ... ");
//            //--- Try to find the Telosys Properties file
//            String sPropFile = Tool.findTelosysPropertiesFile();
//            if (sPropFile != null)
//            {
//                initTelosys(extFactory, sPropFile);
//            }
//            else
//            {
//                $log.error("Telosys.init(extFactory) : cannot found 'telosys.properties' ");
//            }
            TelosysProperties tp = TelosysPropertiesLoader.load();
            initTelosys(extFactory, tp, null );
        }
        return $properties;
    }

    //-----------------------------------------------------------------------------
    /**
     * Initialize the Telosys properties. <br>
     * Try to find a properties file :<br>
     * 1/ specific file name in 'web.xml' ( servlet <init-param>TelosysProperties <init-param>)<br>
     * 2/ 'telosys.properties' located in 'WEB-INF/conf' <br>
     * 3/ 'telosys.properties' located via CLASSPATH <br>
     * 
     * @param extFactory
     * @param config
     * @return
     */
    public static Properties init(IExternalMainFactory extFactory, ServletConfig config)
    {
        if (checkNotInitialized())
        {
            $log.info("INIT : Telosys.init(extFactory, servletConfig) ... ");
//            //--- Try to find the Telosys Properties file
//            String sPropFile = Tool.findTelosysPropertiesFile(config);
//            if (sPropFile != null)
//            {
//                initTelosys(extFactory, sPropFile);
//            }
//            else
//            {
//                $log.error("Telosys.init(extFactory, config) : cannot found 'telosys.properties' ");
//            }
//
            //ServletContext servletContext = config.getServletContext();
            $ServletContext =  config.getServletContext();
                
            TelosysProperties tp = TelosysPropertiesLoader.load($ServletContext);
            initTelosys(extFactory, tp, $ServletContext );

            //--- Set the default Telosys application root directory from the ServletContext
            if ($ServletContext != null)
            {
                $TelosysWebAppRootDir = $ServletContext.getRealPath("/"); // NB : Can be null
                $TelosysWebAppName = $ServletContext.getServletContextName();
            }    
        }
        return $properties;
    }

    //-----------------------------------------------------------------------------
    /**
     * Initialize the Telosys properties from a properties file
     * 
     * @param extFactory
     * @param sPropFile
     * @return
     */
    public static Properties init(IExternalMainFactory extFactory, String sPropFile)
    {
        if (checkNotInitialized())
        {
            $log.info("INIT : Telosys.init(extFactory, file) ... ");
//            if (sPropFile != null)
//            {
//                initTelosys(extFactory, sPropFile);
//            }
//            else
//            {
//                $log.error("Telosys.init(extFactory, file) : properties file is null !");
//            }
            TelosysProperties tp = TelosysPropertiesLoader.load(sPropFile);
            initTelosys(extFactory, tp, null );
        }
        return $properties;
    }

    //-----------------------------------------------------------------------------

    //-----------------------------------------------------------------------------
    // Rcupration des "properties" gnrales de TELOSYS
    //-----------------------------------------------------------------------------
    //-----------------------------------------------------------------------------
    //--- PUBLIC GETTERS/SETTERS
    //-----------------------------------------------------------------------------
//    /**
//     * Returns the full path of the properties file<br>
//     * e.g. : "C:\dir1\dir2\mywebapp\WEB-INF\conf\telosys.properties"
//     * @return
//     */
//    public static String getPropertiesFile()
//    {
//        return $TelosysPropFile ;
//    }
    //-----------------------------------------------------------------------------
//    /**
//     * Returns the Telosys configuration directory ( absolute filesystem directory )<br>
//     * e.g. "C:\dir1\dir2\mywebapp\WEB-INF\conf" <br>
//     * @return
//     */
//    public static String getConfigDirectory()
//    {
//        return $TelosysConfDir ;
//    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the absolute filesystem root directory of the current Web App <br>
     * e.g. "C:\dir1\dir2\mywebapp\" <br> 
     * Set from ServletContext.getRealPath("/"), so it can be null
     * @return the full WebApp path or null if the WAR file is not exploded in the server filesystem
     */
    public static String getWebAppRootDir()
    {
        return $TelosysWebAppRootDir ;
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the name of the current Web App  ( from tag 'display-name' in web.xml ) <br>
     * Set from ServletContext.getServletContextName()
     * @return the WebApp display name or null if not defined
     */
    public static String getWebAppName()
    {
        return $TelosysWebAppName ;
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the SAX parser class name <br>
     * @return
     */
    public static String getSAXParserClassName()
    {
        return $sSAXParserClassName ;
    }
    //-----------------------------------------------------------------------------
    /**
     * Return the full path for the given relative file<br> 
     * ( ie : "myfile.txt" -> "C:/dir1/dir2/mywebapp/myfile.txt" ) <br>
     * This method concats the "Application Root Dir" with the file name<br>
     * if the "Application Root Dir" is not set, it returns the file as is.<br>
     * @param sFile
     * @return 
     */
    public static String getFullPath (String sFile )
    {
        if ( $TelosysWebAppRootDir != null )
        {
            String sRoot2 = $TelosysWebAppRootDir ;
            if ( $TelosysWebAppRootDir.endsWith("/") || $TelosysWebAppRootDir.endsWith("\\") )
            {
                //--- Cut the last char
                sRoot2 = $TelosysWebAppRootDir.substring(0, $TelosysWebAppRootDir.length()-1 ) ;
            }

            if ( sFile != null )
            {
                String sFile2 = sFile ;
                if ( sFile.startsWith("/") )
                {
                    //--- Cut the first char
                    sFile2 = sFile.substring(1); 
                }
                return sRoot2 + "/" + sFile2 ;                                
            }
            else
            {
                return sRoot2 + "/" ;
            }        
            
        }
        else
        {
            Telosys.error("Telosys.getFullPath(" + sFile + ") : WebApp RootDir is null !");
            return sFile ; // No change 
        }
    }
    //-----------------------------------------------------------------------------
//    /**
//     * Transforms the given file name : <br>
//     * replace '.' by the 'config directory' <br>
//     *   ( e.g. './myfile' -> 'C:/dir1/dir2/mywebapp/WEB-INF/conf/myfile' ) <br>
//     * replace '$ROOT/' by the 'web app root directory' 
//     *   ( e.g. '$ROOT/myfile' -> 'C:/dir1/dir2/mywebapp/myfile' ) <br> 
//     * @param sFile
//     * @return
//     */
//    public static String transformFileName(String sFile)
//    {
//        if ( sFile == null )
//        {
//            return null ;
//        }
//        
//        if ( sFile.length() == 0 )
//        {
//            return sFile ;
//        }
//        
//        //--- If the file name starts with "." => it's a relative path 
//        if ( sFile.charAt(0) == '.' )
//        {
//            // ie : "./foo" => "Dir/foo"
//            return $TelosysConfDir + sFile.substring(1) ;
//        }
//        
//        //--- If the file name starts with "$ROOT" => it's a "WEB APP ROOT" path 
//        if ( sFile.startsWith("$ROOT/") )
//        {
//            if ( $TelosysWebAppRootDir != null )
//            {
//                // ie : "$ROOT/foo" => "WebAppRootDir/foo"
//                return $TelosysWebAppRootDir + sFile.substring(6) ;            
//            }
//            else
//            {
//                Telosys.error("Telosys.transformFileName(" + sFile + ") : WebApp RootDir is null !");
//                return sFile.substring(6) ;            
//            }
//        }
//
//        //--- Not special path => return "as is"
//        return sFile ;
//    }

    //-----------------------------------------------------------------------------
    /**
     * Returns the Telosys properties ( loaded from the 'telosys.properties' file )
     * @return
     */
    public static Properties getProperties()
    {
        return $properties;
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the Telosys factory
     * @return
     */
    public static TelosysFactory getFactory()
    {
        return $telosysFactory;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns the XmlWrapperProvider provided by the Telosys factory
     * @return
     */
    public static IXmlWrapperProvider getXmlWrapperProvider()
    {
        if ( $telosysFactory != null )
        {
            return $telosysFactory.getXmlWrapperProvider();
        }
        return null ;
    }
    
    //-----------------------------------------------------------------------------
    /**
     * Returns true if user authentication is required
     * @return
     */
    public static boolean isAuthenticationRequired()
    {
        return $bAuthenticationRequired ;
    }
    //-----------------------------------------------------------------------------
    public static DocumentBuilderFactory getXmlDomParserFactory()
    {
//        // NB: An implementation of the DocumentBuilderFactory class is NOT guaranteed to be thread safe.
//        //     => a factory is created for each call 
//        String sParserClass = null ;
//        if ( $sXmlDomParser != null )
//        {
//            sParserClass = $sXmlDomParser ;
//        }
//        else
//        {
//            //--- Use CRIMSON by default
//            sParserClass = DOM_CRIMSON ;            
//        }
//        //--- Set the JAXP system propety 
//        System.setProperty( DOM_PARSER, sParserClass );
        //--- Create a new factory instance via JAXP
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        //--- Configuring the Factory ...
        //factory.setValidating(true);
        //factory.setNamespaceAware(true);        

        $log.info("Telosys.getXmlDomParserFactory() : JAXP factory class = " + factory.getClass().getName() );
        
        return factory ;
    }
    
    //-----------------------------------------------------------------------------
    //--- PUBLIC LOG METHODS
    //-----------------------------------------------------------------------------
    public static void error(String sClassName, String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.error(sClassName, sMsg);
        }
        else
        {
            Console.error(sClassName, sMsg);
        }
    }

    public static void error(String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.error(sMsg);
        }
        else
        {
            Console.error(sMsg);
        }
    }

    public static String getMsg(Throwable ex)
    {
        StackTraceElement t[] = ex.getStackTrace();
        StackTraceElement loc = t[0];
        return "Exception " + ex.toString() + " in file '" + loc.getFileName() + "', method '" + loc.getMethodName() + "', line " + loc.getLineNumber();
    }

    public static String getCaller(StackTraceElement elt)
    {
        return elt.getClassName() + "." + elt.getMethodName() + "():" + elt.getLineNumber();
    }

    public static String getCallers(Throwable ex)
    {
        StackTraceElement t[] = ex.getStackTrace();
        StackTraceElement loc = null;
        StringBuffer sbMsg = new StringBuffer(300);
        sbMsg.append("Exception : callers : ");
        for ( int i = 1 ; i < t.length && i < 3 ; i++ )
        {
            loc = t[i];
            if (i > 1)
                sbMsg.append(" <- ");
            //sbMsg.append(loc.toString());
            sbMsg.append(getCaller(loc));
        }
        return sbMsg.toString();
    }

    public static void error(String sClassName, Throwable ex)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.error(sClassName, getMsg(ex));
            $TelosysLogger.error(sClassName, getCallers(ex));
        }
        else
        {
            Console.error(sClassName, getMsg(ex));
            Console.error(sClassName, getCallers(ex));
        }
    }

    public static void error(Throwable ex)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.error(getMsg(ex));
            $TelosysLogger.error(getCallers(ex));
        }
        else
        {
            Console.error(getMsg(ex));
            Console.error(getCallers(ex));
        }
    }

    //-----------------------------------------------------------------------------
    public static void warn(String sClassName, String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.warn(sClassName, sMsg);
        }
        else
        {
            Console.warn(sClassName, sMsg);
        }
    }

    public static void warn(String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.warn(sMsg);
        }
        else
        {
            Console.warn(sMsg);
        }
    }

    //-----------------------------------------------------------------------------
    public static void info(String sClassName, String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.info(sClassName, sMsg);
        }
        else
        {
            Console.info(sClassName, sMsg);
        }
    }

    public static void info(String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.info(sMsg);
        }
        else
        {
            Console.info(sMsg);
        }
    }

    //-----------------------------------------------------------------------------
    public static void trace(String sClassName, String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.trace(sClassName, sMsg);
        }
        else
        {
            Console.trace(sClassName, sMsg);
        }
    }

    //-----------------------------------------------------------------------------

    public static void trace(String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.trace(sMsg);
        }
        else
        {
            Console.trace(sMsg);
        }
    }
    
    //=============================================================================
    // Acces to other configuration files/resources
    //=============================================================================
    
    //-----------------------------------------------------------------------------
    // dbconfig.xml
    //-----------------------------------------------------------------------------
    /**
     * Returns the file name (or resource name) of the DbConfig XML file.
     * @return
     */
    public static String getDbConfigFileName()
    {
        return getConfigFile("DbConfFile");
//        String s = getProperty("DbConfFile");
//        return ( s != null ? s.trim() : null ) ;
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the InputStream of the DbConfig XML file or resource.
     * 
     * @return the InputStream ( or null if cannot get it )
     */
    public static InputStream getDbConfigInputStream()
    {
        String s = getDbConfigFileName() ;
        return ( s != null ? getInputStream(s) : null ) ;
    }
    
    //-----------------------------------------------------------------------------
    // screens.xml
    //-----------------------------------------------------------------------------
    /**
     * Returns the file name (or resource name) of the DbConfig XML file.
     * @return
     */
    public static String getScreensConfigFileName()
    {
        return getConfigFile("ScreensConfFile");
//        String s = getProperty("ScreensConfFile");
//        return ( s != null ? s.trim() : null ) ;
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the InputStream of the DbConfig XML file or resource.
     * 
     * @return the InputStream ( or null if cannot get it )
     */
    public static InputStream getScreensConfigInputStream()
    {
        String s = getScreensConfigFileName() ;
        return ( s != null ? getInputStream(s) : null ) ;
    }

    //-----------------------------------------------------------------------------
    // tags.properties
    //-----------------------------------------------------------------------------
    public static String getTagsConfigFileName()
    {
        return getConfigFile("TagsConfFile");
//        String s = getProperty("TagsConfFile");
//        return ( s != null ? s.trim() : null ) ;
    }
    //-----------------------------------------------------------------------------
    public static InputStream getTagsConfigInputStream()
    {
        String s = getTagsConfigFileName() ;
        return ( s != null ? getInputStream(s) : null ) ;
    }
    
    //-----------------------------------------------------------------------------
    // traces.properties
    //-----------------------------------------------------------------------------
    public static String getTraceConfigFileName()
    {
        return getConfigFile("TraceConfFile");
//        String s = getProperty("TraceConfFile");
//        return ( s != null ? s.trim() : null ) ;
    }
    //-----------------------------------------------------------------------------
    public static InputStream getTraceConfigInputStream()
    {
        String s = getTraceConfigFileName() ;
        return ( s != null ? getInputStream(s) : null ) ;
    }
    
    //-----------------------------------------------------------------------------
    //-----------------------------------------------------------------------------
    private static String getConfigFile(String sPropertyName)
    {
        if ( $TelosysProperties != null )
        {
            return $TelosysProperties.getConfigFile(sPropertyName);
        }
        else
        {
            error("getConfigFile('"+sPropertyName+"') : TelosysProperties == null ");
            return null ;
        }
    }
    //-----------------------------------------------------------------------------
//    private static String getProperty(String s)
//    {
//        if ( $properties != null )
//        {
//            String v = $properties.getProperty(s);
//            if ( v == null )
//            {
//                error("Property '" + s + "' not found in Telosys properties !");
//            }
//            return v;
//        }
//        else
//        {
//            error("Cannot get property '" + s + "' because Telosys properties are not initialized.");
//        }
//        return null ;
//    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the resource file InputStream using the same origin as the main Telosys properties
     * @param sResource resource or file name
     * @return
     */
    private static InputStream getInputStream(String sResource)
    {
        InputStream is = TelosysPropertiesLoader.getInputStream(sResource, $iTelosysPropertiesOrigin, $ServletContext );
        if ( is == null )
        {
            $log.error("getInputStream('" + sResource + "') : return NULL" );
        }
        return is;
    }
    //-----------------------------------------------------------------------------
}