package org.objectweb.telosys.common; 

import java.util.Date;
import java.util.Properties;

/**
 * @author Laurent GUERIN
 *
 */
public class TelosysProperties
{
    public final static int UNKNOWN = 0 ;
    public final static int FILESYSTEM = 1 ;
    public final static int WEBAPP = 2 ;
    public final static int CLASSPATH = 3 ;
    
    private Properties _prop = null ;
    private String     _sFile = null ; 
    //private String     _sLocation = null ; // Configuration files directory 
    private int        _iOrigin = UNKNOWN ;
    private Date       _initDate = null ;
    
    //-----------------------------------------------------------------------------------------
    /**
     * @param prop
     * @param file
     * @param origin
     */
    protected TelosysProperties(Properties prop, String file, int origin)
    {
        super();
        _prop = prop;
        _sFile = file;
        if ( origin >= 1 && origin <= 3 )
        {
            _iOrigin = origin;
        }
        _initDate = new Date();
    }
    
    
    //-----------------------------------------------------------------------------------------
    public int getOrigin()
    {
        return _iOrigin;
    }
    
    public Properties getProperties()
    {
        return _prop;
    }
    
    public String getFile()
    {
        return _sFile;
    }

    public Date getInitDate()
    {
        return _initDate;
    }
    
    //-----------------------------------------------------------------------------------------
    private String getLocation()
    {
        if ( _sFile != null )
        {            
	        int i1 = _sFile.lastIndexOf('/');
	        int i2 = _sFile.lastIndexOf('\\');
	        int i = ( i1 > i2 ? i1 : i2 ) ; // Take the Max
	        if ( i >= 0 )
	        {
	            return _sFile.substring(0,i); // Root without ending '/' or '\'
	        }
	        else
	        {
	            return "" ;
	        }
        }
        return "";
    }
    //-----------------------------------------------------------------------------------------
    private String transformFileName(String sFile)
    {
        if ( sFile == null )
        {
            return null ;
        }
        
        if ( sFile.length() == 0 )
        {
            return sFile ;
        }
        
        //--- If the file name starts with "." => it's a relative path 
        if ( sFile.charAt(0) == '.' )
        {
            // ie : "./foo" => "Dir/foo"
            return getLocation() + sFile.substring(1) ;
        }
        //--- Not special path => return "as is"
        return sFile ;
    }
    
    //-----------------------------------------------------------------------------------------
    public String getConfigFile(String sParamName)
    {
        if ( _prop != null )
        {
            String sTraceConfFile = _prop.getProperty(sParamName);
            if (sTraceConfFile != null)
            {
                sTraceConfFile = sTraceConfFile.trim() ;
                return transformFileName(sTraceConfFile);
            }
        }
        return null;
    }

    //-----------------------------------------------------------------------------------------
    // Other configuration files 
    //-----------------------------------------------------------------------------------------
    public String getTracePropertiesFile()
    {
        return getConfigFile("TraceConfFile");
    }
    //-----------------------------------------------------------------------------------------
//    public String getTagsPropertiesFile()
//    {
//        return getConfigFile("TagsConfFile");
//    }
//    //-----------------------------------------------------------------------------------------
//    public String getDbConfFile()
//    {
//        return getConfigFile("DbConfFile");
//    }
//    //-----------------------------------------------------------------------------------------
//    public String getScreensConfFile()
//    {
//        return getConfigFile("ScreensConfFile");
//    }
    
    //-----------------------------------------------------------------------------------------
    public String toString()
    {
        String s = "unknown" ;
        switch(_iOrigin)
        {
            case FILESYSTEM : s = "file system" ; break ;
            case WEBAPP : s = "web application" ; break ;
            case CLASSPATH : s = "class path" ; break ;
        }
        return "Telosys properties loaded from '"+_sFile+"' ( origin : " + s + " ) at " + _initDate ; 
    }
    //-----------------------------------------------------------------------------------------
}
