package org.objectweb.telosys.rpl.xml;

import java.util.HashMap;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.rpl.TelosysRPL;

/**
 * XML Wrappers Registry to provide XML Wrappers 
 * 
 * @author Laurent GUERIN
 *  
 */
public class StandardXmlWrapperRegistry extends TelosysObject implements IXmlWrapperProvider
{
//    //-----------------------------------------------------------------------------------
//    // Inner Class
//    //-----------------------------------------------------------------------------------
//    private static class GenericListWrapper
//    {
//        private Class       _dlwClass      = null;
//
//        private IXmlWrapper _beanWrapper   = null;
//
//        public GenericListWrapper( Class dlwClass, IXmlWrapper beanWrapper)
//        {
//            _dlwClass = dlwClass;
//            _beanWrapper = beanWrapper;
//        }
//
//        public Class getDataListWrapperClass()
//        {
//            return _dlwClass;
//        }
//
//        public IXmlWrapper getBeanWrapper()
//        {
//            return _beanWrapper;
//        }
//    }

    //-----------------------------------------------------------------------------------
    //private static final TelosysClassLogger log                           = new TelosysClassLogger(StandardXmlWrapperRegistry.class);

    //--- SPECIFIC WRAPPERS ( for VO beans )
//    private final static int                NB_SPECIFIC                   = 40;

//    private final static HashMap            $hmSpecificWrappersByClass    = new HashMap(NB_SPECIFIC);

    private final static HashMap            $hmWrappersByBeanClass    = new HashMap(128);

    //--- GENERIC WRAPPERS ( for VO lists )
//    private final static int                NB_GENERIC                    = 30;

//    private final static HashMap            $hmGenericListWrappersByClass = new HashMap(NB_GENERIC);

    //-----------------------------------------------------------------------------------
    /**
     * Registers a specific XML wrapper class 
     * 
     * @param beanWrapperClass :
     *            the wrapper class
     */
//    protected static void register(Class beanWrapperClass)
//    {
//        if (IXmlWrapper.class.isAssignableFrom(beanWrapperClass) != true)
//        {
//            log.error("register(" + beanWrapperClass.getName() + ") : this class does not implements IXmlWrapper !");
//            return ;
//        }
//
//        //--- Try to get class of the bean
//        Class beanClass = null;
//        try
//        {
//            Object oValue = ClassUtil.getFieldValue(beanWrapperClass, "BEAN_CLASS");
//            if (oValue instanceof Class)
//            {
//                beanClass = (Class) oValue;
//            }
//        } catch (TelosysException e)
//        {
//            beanClass = null;
//        }
//
//        if (beanClass != null)
//        {
//            //--- Register the wrapper by bean class
//            $hmSpecificWrappersByClass.put(beanClass.getName(), beanWrapperClass);
//        }
//        else
//        {
//            log.error("register(" + beanWrapperClass.getName() + ") : Cannot get field BEAN_CLASS !");
//        }
//        return ;
//    }

    protected static void register(Class beanWrapperClass)
    {
        if ( beanWrapperClass == null )
        {
            throw new TelosysRuntimeException("register(null) : class argument is null ");
        }
        
        if (IXmlWrapper.class.isAssignableFrom(beanWrapperClass) != true)
        {
            throw new TelosysRuntimeException("register("+beanWrapperClass+") : does not implement IXmlWrapper !");
        }

        //--- Create the Wrapper instance 
        IXmlWrapper wrapperInstance = createWrapperInstance(beanWrapperClass);
        
        //--- Register the Wrapper instance 
        registerInstance(wrapperInstance);
    }
    
    private static void registerInstance(IXmlWrapper wrapperInstance)
    {
        if (wrapperInstance == null)
        {
            throw new TelosysRuntimeException("registerInstance(null) : no parameter (null)");
        }
        Class beanClass = wrapperInstance.getBeanClass();
        
//        //--- Try to get class of the bean
//        Class beanClass = null;
//        try
//        {
//            Object oValue = ClassUtil.getFieldValue(beanWrapperClass, "BEAN_CLASS");
//            if (oValue instanceof Class)
//            {
//                beanClass = (Class) oValue;
//            }
//        } catch (TelosysException e)
//        {
//            beanClass = null;
//        }

        if (beanClass != null)
        {
            //--- Register the wrapper by bean class
            //$hmWrappersByBeanClass.put(beanClass.getName(), beanWrapperClass);
            $hmWrappersByBeanClass.put(beanClass.getName(), wrapperInstance);
        }
        else
        {
            //log.error("register(" + beanWrapperClass.getName() + ") : Cannot get bean class !");
            throw new TelosysRuntimeException("registerInstance(" + wrapperInstance.getClass().getName() + ") : Cannot get bean class");
        }
        return ;
    }
    //-----------------------------------------------------------------------------------
//    /**
//     * Register a specific XML wrapper class for a specific VO bean with a VO list <br>
//     * (the xml tag is given by the static field 'XML_TAG')
//     * @param beanWrapperClass : the class of the XML wrapper
//     * @param sListTagName : the tag name of the VO List 
//     * @param listClass : the VO List class
//     */
//    protected static void register(Class beanWrapperClass, Class listClass)
//    {
//        //--- Check arguments
//        if (beanWrapperClass == null)
//        {
//            log.error("register(.,.,.) : The bean wrapper class is null !");
//            return;
//        }
////        if (listClass == null)
////        {
////            log.error("register(.,.,.) : The VOList class is null !");
////            return;
////        }
//
//        //-----------------------------------------------------------------------------------
//        //--- 1) Register the bean xml wrapper 
//        //-----------------------------------------------------------------------------------
//        register(beanWrapperClass);
//
////        //-----------------------------------------------------------------------------------
////        //--- 2) Register the VOList XML wrapper
////        //-----------------------------------------------------------------------------------
////        //--- Create the bean (list element) wrapper
////        IXmlWrapper beanWrapper = createWrapperInstance(beanWrapperClass);
////        //--- Register the generic wrapper definition
////        GenericListWrapper glw = new GenericListWrapper( listClass, beanWrapper);
////        //--- For access by Class (the key is the name of the VOList(DataListWrapper) class)
////        $hmGenericListWrappersByClass.put(listClass.getName(), glw);
//    }

    //-----------------------------------------------------------------------------------
    /**
     * Creates a wrapper instance with the given class
     * 
     * @param objClass
     *            class to use to create a wrapper instance
     * @return the wrapper instance (never null)
     */
    private static IXmlWrapper createWrapperInstance(Class wrapperClass)
    {
        //log.trace("createWrapperInstance(" + wrapperClass + ")...");
        if (wrapperClass != null)
        {
            try
            {
                //--- Create the wrapper instance with the given class
                Object objWrapper = wrapperClass.newInstance();
                if (objWrapper instanceof IXmlWrapper)
                {
                    //log.trace("createWrapperInstance(" + wrapperClass + ") : OK, wrapper inctance created");
                    return (IXmlWrapper) objWrapper;
                }
                else
                {
                    throw new TelosysRuntimeException("createWrapperInstance(" + wrapperClass + ") : not a wrapper class ");
                }
            } catch (InstantiationException ex)
            {
                throw new TelosysRuntimeException("createWrapperInstance(" + wrapperClass + ") : InstantiationException ", ex);
                //log.error("createWrapperInstance(" + objClass + ") : InstantiationException ");
            } catch (IllegalAccessException ex)
            {
                throw new TelosysRuntimeException("createWrapperInstance(" + wrapperClass + ") : IllegalAccessException ", ex);
                //log.error("createWrapperInstance(" + objClass + ") : IllegalAccessException ");
            } catch (Throwable t)
            {
                throw new TelosysRuntimeException("createWrapperInstance(" + wrapperClass + ") : Throwable " + t.toString(), t);
                //log.error("createWrapperInstance(" + objClass + ") : Throwable " + t.toString());
            }
        }
        else
        {
            throw new TelosysRuntimeException("createWrapperInstance(null) : class is null");
        }
    }

    //-----------------------------------------------------------------------------------
    /**
     * Try to load a Wrapper class from the given bean class name 
     * ( wrapper class name = "BeanClass" + "WrapperSuffix" )
     * @param psBeanClassName
     * @return the class loaded or null if not found
     */
    private Class loadDefaultWrapperClass(String psBeanClassName)
    {
        trace("loadDefaultWrapperClass(" + psBeanClassName + ")");
        
//        String sWrapperPackage = "demo.xmlwrapper" ;
//        String sWrapperSuffix  = "XmlWrapper" ;

        //--- Wrapper class package
        String sWrapperPackage = TelosysRPL.getXmlWrapperPackage() ;
        if ( sWrapperPackage == null )
        {
            return null ;  // No package = no default wrapper => NOT FOUND        
        }
        if ( ! sWrapperPackage.endsWith("."))
        {
            sWrapperPackage = sWrapperPackage + "." ;
        }

        //--- Wrapper class suffix
        String sWrapperSuffix  = TelosysRPL.getXmlWrapperSuffix() ;
        if ( sWrapperSuffix == null )
        {
            sWrapperSuffix = "" ;  // No suffix         
        }
        
        
        String[] r = psBeanClassName.split("\\.");
        int n = r.length ;
        if ( n > 0 )
        {
            String s = r[n-1];
            if ( s != null )
            {
                Class clWrapper = null ;
                String sWrapperClassName = sWrapperPackage + s + sWrapperSuffix ;                
                trace("loadDefaultWrapperClass(" + psBeanClassName + ") : class name = '" + sWrapperClassName + "'" );
                try
                {
                    clWrapper = Class.forName(sWrapperClassName);
//                    //--- Class found => register it for future use 
//                    register(clWrapper);
                } catch (ClassNotFoundException ex)
                {
                    //--- Class not found : it's not an error ( there's no wrapper for this kind of object )
                    clWrapper = null ; // NOT FOUND
                    //trace("loadDefaultWrapperClass(" + psBeanClassName + ") : class name = '" + sWrapperClassName + "' not found " );
	            } catch (Throwable t)
	            {
                    //--- Other error cases : abnormal => ERROR
	                throw new TelosysRuntimeException("cannot load XML wrapper class '" + sWrapperClassName + "' : Throwable " + t.toString(), t);
	                //log.error("createWrapperInstance(" + objClass + ") : Throwable " + t.toString());
	            }
                return clWrapper ;
            }
        }
        return null ;
    }
    
    //-----------------------------------------------------------------------------------
    /**
     * Try to find the XML Wrapper for the given Java class 
     * Search in the wrapper registry, if not found try to load 
     * the default wrapper class  
     * @param beanClass 
     * @return the XML Wrapper ( or null if wrapper not found )
     */
    private IXmlWrapper getSpecificWrapper(Class beanClass)
    {
        if (beanClass == null)
        {
            error("getSpecificWrapper() : class argument is null ");
            return null;
        }
        String sClassName = beanClass.getName();
        trace("getSpecificWrapper(" + sClassName + ")...");
        //--- Search a wrapper in the registry 
        Object obj = $hmWrappersByBeanClass.get(sClassName);
        
        if (obj != null)
        {
            //--- Wrapper FOUND in the registry => use it  
            trace("getSpecificWrapper(" + sClassName + ") : CLASS FOUND");   
            if ( obj instanceof IXmlWrapper )
            {
                return (IXmlWrapper) obj ;
            }
            else
            {
                throw new TelosysRuntimeException("registry corrupted : wrapper for '"+sClassName+"' is not a IXmlWrapper ");
            }
        }
        
        //--- Wrapper NOT FOUND in the registry => try to load the default wrapper class  
        trace("getSpecificWrapper(" + sClassName + ") : CLASS NOT FOUND => TRY TO LOAD DEFAULT CLASS");            
        Class wrapperClass = loadDefaultWrapperClass(sClassName);
        if (wrapperClass != null)
        {
            //--- Create the wrapper instance 
            IXmlWrapper wrapper = createWrapperInstance(wrapperClass);
            //--- Register it to reuse it in the future 
            registerInstance(wrapper);
            return wrapper ;
        }
        else
        {
	        trace("getSpecificWrapper(" + sClassName + ") : wrapper not found");
	        return null;
        }
    }

    //-----------------------------------------------------------------------------------
    // Try to find and return the generic list XML wrapper associated with the tag name
    //-----------------------------------------------------------------------------------
//    private IXmlWrapper createGenericWrapperInstance(GenericListWrapper glw, String sKey)
//    {
//        trace("createGenericWrapperInstance(x) ...");
//        if (glw != null)
//        {
//            //--- Create the list wrapper
//            IXmlWrapper theListWrapper = new GenericVOListXmlWrapper( glw.getDataListWrapperClass(), glw.getBeanWrapper());
//            trace("getGenericListWrapper(" + sKey + ") : OK, wrapper found ");
//            return theListWrapper;
//        }
//        else
//        {
//            trace("getGenericListWrapper(" + sKey + ") : wrapper definition not found ");
//            return null;
//        }
//    }

//    private IXmlWrapper getGenericListWrapper(Class cl)
//    {
//        if (cl == null)
//        {
//            error("getGenericListWrapper() : class argument is null ");
//            return null;
//        }
//        String sClassName = cl.getName();
//        trace("getGenericListWrapper(" + sClassName + ")...");
//        //--- Search a Generic List Wrapper for this Class Name
//        GenericListWrapper glw = (GenericListWrapper) $hmGenericListWrappersByClass.get(sClassName);
//        return createGenericWrapperInstance(glw, sClassName);
//    }

    //-----------------------------------------------------------------------------------
    // IXmlWrapperProvider implementation
    //-----------------------------------------------------------------------------------
    
    //-----------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.rpl.xml.IXmlWrapperProvider#getWrapper(java.lang.Class)
     */
    public IXmlWrapper getWrapper(Class beanClass) //throws TelosysException
    {
        if (beanClass == null)
        {
            throw new TelosysRuntimeException("getWrapper(Class c) : class argument is null ");
        }
        trace("getWrapper( class [" + beanClass.getName() + "] )");
        IXmlWrapper wrapper = null;
        //--- 1st : try to find a Specific Wrapper
        wrapper = getSpecificWrapper(beanClass);
        if (wrapper == null)
        {
//            //--- 2nd : try to find an Generic List Wrapper
//            wrapper = getGenericListWrapper(cl);
//            if (wrapper == null)
//            {
//                trace("wrapper not found");
//            }
            trace("wrapper not found");
        }
        return wrapper;
    }

    //-----------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.rpl.xml.IXmlWrapperProvider#getWrapper(java.lang.Object)
     */
    public IXmlWrapper getWrapper(Object beanInstance) //throws TelosysException
    {
        trace("getWrapper(bean)");
        if (beanInstance == null)
        {
            throw new TelosysRuntimeException("getWrapper(Object o) : object argument is null ");
        }
        Class cl = beanInstance.getClass();
        return getWrapper(cl);
    }

    //-----------------------------------------------------------------------------------
}