package org.objectweb.telosys.screen.core;

import java.util.Hashtable;

import javax.servlet.http.HttpSession;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.screen.env.ScreenApplication;
import org.objectweb.telosys.screen.env.ScreenSession;

/**
 * The standard ScreenContext implementation.
 * 
 * @author Laurent GUERIN
 *  
 */
public class StandardScreenContext extends TelosysObject implements ScreenContext
{
    private String             _sScreenName        = null;

    private int                _iScreenId          = 0;  

    private ScreenSession      _screenSession      = null;

    private ScreenData         _currentScreenData  = null;

    private ScreenData         _screenDataImage    = null;

    private ScreenActions      _screenActions      = null;

    private ScreenTriggers     _screenTriggers     = null;

    private ScreenProcedures   _screenProcedures   = null;

    private ScreenDataAccessor _screenDataAccessor = null;

    private Class              _screenDataClass    = null;

    //--- Screen Data used for LOAD operation
    private ScreenData         _newDataLoaded      = null;

    //--- Reusable temporary Screen Data (used only for DELETE operations)
    private ScreenData         _tempData           = null;

    //--- Screen context attributes
    private Hashtable          _htAttributes       = new Hashtable();

    /**
     * StandardScreenContext constructor
     * 
     * @param sScreenName :
     *            the symbolic screen name
     * @param screenDataClass :
     *            the class of the screen data
     * @param screenSession :
     *            the screen session
     * @param iScreenId :
     *            the screen ID ( 0 to N )
     */
    public StandardScreenContext(String sScreenName, Class screenDataClass, ScreenSession screenSession, int iScreenId) // v 0.9.4
    {
        _currentScreenData = null; // null : no ScreenData
        _sScreenName = sScreenName;
        _screenDataClass = screenDataClass;
        _screenSession = screenSession;
        //--- Set the Screen ID // ADD v 0.9.4
        if (_iScreenId >= 0)
        {
            _iScreenId = iScreenId;
        }
        else
        {
            error("ScreenContext creation : initScreen : invalid Screen ID : " + iScreenId);
            _iScreenId = 0;
        }
    }

    //---------------------------------------------------------------------------
    // get the screen name
    //---------------------------------------------------------------------------
    public String getScreenName()
    {
        return _sScreenName;
    }

    public int getScreenId()
    {
        return _iScreenId;
    }

    //---------------------------------------------------------------------------
    public ScreenApplication getScreenApplication()
    {
        if (_screenSession != null)
        {
            return _screenSession.getScreenApplication();
        }
        else
        {
            error("getScreenApplication() : ScreenSession instance is null !");
            return null;
        }
    }

    //---------------------------------------------------------------------------
    public ScreenSession getScreenSession()
    {
        return _screenSession;
    }

    //---------------------------------------------------------------------------
    public HttpSession getHttpSession()
    {
        if (_screenSession != null)
        {
            return _screenSession.getHttpSession();
        }
        else
        {
            error("getHttpSession() : ScreenSession instance is null !");
            return null;
        }
    }

    //---------------------------------------------------------------------------
    // get/set/clear the current data of the screen
    //---------------------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getCurrentData()
     */
    public ScreenData getCurrentData()
    {
        return _currentScreenData;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getOrCreateCurrentData()
     */
    public ScreenData getOrCreateCurrentData()
    {
        if (_currentScreenData == null)
        {
            //--- No current data => create it
            _currentScreenData = createNewScreenData();
        }
        return _currentScreenData;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#setCurrentData(org.objectweb.telosys.screen.core.ScreenData)
     */
    public void setCurrentData(ScreenData data)
    {
        _currentScreenData = data;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#clearCurrentData()
     */
    public void clearCurrentData()
    {
        _currentScreenData = null;
    }

    /**
     * Inject DATA CLASS dependency
     * 
     * @param screenDataClass
     */
    /* package */void setScreenDataClass(Class screenDataClass)
    {
        _screenDataClass = screenDataClass;
    }

    /**
     * Injects the ScreenDataAccessor dependency ( v 0.9.5 )
     * @param screenDataAccessor
     */
    /* package */ void setScreenDataAccessor(ScreenDataAccessor screenDataAccessor)
    {
        _screenDataAccessor = screenDataAccessor;
    }

    /**
     * Injects ACTIONS dependency
     * 
     * @param screenActions
     */
    /* package */void setScreenActions(ScreenActions screenActions)
    {
        _screenActions = screenActions;
    }

    /**
     * Inject TRIGGERS dependency
     * 
     * @param screenTriggers
     */
    /* package */void setScreenTriggers(ScreenTriggers screenTriggers)
    {
        _screenTriggers = screenTriggers;
    }

    /**
     * Inject PROCEDURES dependency
     * 
     * @param screenProcedures
     */
    /* package */void setScreenProcedures(ScreenProcedures screenProcedures)
    {
        _screenProcedures = screenProcedures;
    }

    //==============================================================================
    // ABSTRACT METHODS IMPLEMENTATIONS
    //==============================================================================

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getScreenActions()
     */
    public ScreenActions getScreenActions()
    {
        return _screenActions;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getScreenTriggers()
     */
    public ScreenTriggers getScreenTriggers()
    {
        return _screenTriggers; // or null if no specific triggers
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getScreenProcedures()
     */
    public ScreenProcedures getScreenProcedures()
    {
        return _screenProcedures; // or null if no specific triggers
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenContext#getScreenDataAccessor()
     */
    public ScreenDataAccessor getScreenDataAccessor()
    {
        return _screenDataAccessor ;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#createNewScreenData()
     */
    public ScreenData createNewScreenData()
    {
        ScreenData screenData = null;
        //--- If the class to build screen data is set...
        if (_screenDataClass != null)
        {
            //--- Try to create a valid instance of ScreenData with the given class
            try
            {
                //--- Create the screenData instance with the class
                Object object = _screenDataClass.newInstance();
                if (object instanceof ScreenData)
                {
                    screenData = (ScreenData) object;
                }
            } catch (InstantiationException e)
            {
                screenData = null;
                error("createNewScreenData() : InstantiationException ! ", e);
            } catch (IllegalAccessException e)
            {
                screenData = null;
                error("createNewScreenData() : IllegalAccessException ! ", e);
            }
        }
        return screenData;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#beginAction()
     */
    public ScreenData beginAction()
    {
        //--- Prepare a "ScreenData image" (copy of the "current ScreenData" )
        if (_currentScreenData == null)
        {
            //--- No "current ScreenData", so ...
            // No copy of the original ScreenData
            _screenDataImage = null;
            // New instance of "current ScreenData"
            _currentScreenData = createNewScreenData();
        }
        else
        {
            //--- The "current ScreenData" exists, so ...
            // Ensure that there's a "ScreenData image" ready to use
            if (_screenDataImage == null)
            {
                _screenDataImage = createNewScreenData();
            }
            // Copy the "current ScreenData" in the "ScreenData image"
            //_currentScreenData.copyElementsTo(_screenDataImage);
            _currentScreenData.copyTo(_screenDataImage); // v 0.9.8 ( 08/01/08 )
        }
        //--- Return the "current ScreenData"
        return _currentScreenData;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#cancelAction()
     */
    public void cancelAction()
    {
        // _currentScreenData = _screenDataImage; // v 0.9.8 ( 08/01/08 )
        //--- Restore the preserved "ScreenData image" by reverse copy // v 0.9.8 ( 08/01/08 )
        if (_screenDataImage == null)
        {
            //--- Restore the original "null value"
            _currentScreenData = null ; 
        }
        else
        {
            //--- Restore the original state
            _screenDataImage.copyTo(_currentScreenData);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getDataForLoad(org.objectweb.telosys.rpl.ScreenRequest)
     */
    public ScreenData getDataForLoad() //throws TelosysException
    {
        trace("getDataForLoad");
        
//      v 0.9.8 ( 08/01/08 ) : re-use and initialize Data For Load
        //_newDataLoaded = createNewScreenData();

        // Ensure that there's a "ScreenData instance" ready to use
        if (_newDataLoaded == null)
        {
            _newDataLoaded = createNewScreenData();
        }
        
        // Copy the "current ScreenData" in the "ScreenData for load"
        if (_currentScreenData != null)
        {
            _currentScreenData.copyTo(_newDataLoaded); 
        }
        // Reset the "found flag" to "not found"
        _newDataLoaded.setNotFound();
        
        //--- Return the data
        return _newDataLoaded;
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenContext#getTempData()
     */
    public ScreenData getTempData() 
    {
        trace("getTempData");
        //--- Create or reuse a working ScreenData 
        if (_tempData == null)
        {
            _tempData = createNewScreenData();
        }
        else
        {
            //_tempData.clearElements();
            _tempData.clear(); // v 0.9.8 ( 08/01/08 )
        }

        //--- Return the data
        return _tempData;
    }

    //---------------------------------------------------------------------------
    // Screen Context attributes
    //---------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenContext#setAttribute(java.lang.String, java.lang.Object)
     */
    public Object setAttribute(String name, Object value)
    {
        //--- Avoid NullPointerException if key is null
        if (name != null && value != null)
        {
            return _htAttributes.put(name, value);
        }
        else
        {
            error("setAttribute(name,value) : name or value is null !");
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenContext#getAttribute(java.lang.String)
     */
    public Object getAttribute(String name)
    {
        //--- Avoid NullPointerException if key is null
        if (name != null)
        {
            return _htAttributes.get(name);
        }
        else
        {
            error("getAttribute(name) : name is null !");
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenContext#removeAttribute(java.lang.String)
     */
    public Object removeAttribute(String name)
    {
        //--- Avoid NullPointerException if key is null
        if (name != null)
        {
            return _htAttributes.remove(name);
        }
        else
        {
            error("removeAttribute(name) : name is null !");
            return null;
        }
    }
}