package org.objectweb.telosys.uil.taglib;

import javax.servlet.jsp.PageContext;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.data.DataRow;
import org.objectweb.telosys.util.BeanUtil;

/**
 * Static methods to retrieve informations from the "current bean"
 * 
 * @author Laurent GUERIN
 *  
 */
public class CurrentBean
{

    private static int getColumn(String sCol, DataRow dr)
    {
        int iCol = 0;
        try
        {
            iCol = Integer.parseInt(sCol);
            if (iCol <= 0)
            {
                return -1;
            }
            if (iCol > dr.getSize())
            {
                return -2;
            }
        } catch (NumberFormatException e)
        {
            return -3;
        }
        return iCol;
    }

    //----------------------------------------------------------------------------------
    /**
     * Return the value of a column from the "current bean" <br>
     * ( usefull for a DataRow current bean ) 
     * @param pageContext
     * @param sCol
     * @return
     */
    public static Object getColumn ( PageContext pageContext, String sCol )
    {
        Object oBean = Page.getCurrentBean(pageContext);
        if (oBean instanceof DataRow)
        {
            DataRow dr = (DataRow) oBean;
            int iCol = getColumn(sCol, dr);
            if (iCol > 0 && iCol <= dr.getSize())
            {
                return dr.getObject(iCol) ;
            }
            else
            {
                return "(col='" + sCol + "':invalid)" ;
            }
        }
        else
        {
            return "(no DataRow)" ;
        }
    }

    //----------------------------------------------------------------------------------
    /**
     * Return the value of an attribute from the "current bean" <br>
     * ( usefull for a "standard bean" current bean ) 
     * @param pageContext
     * @param sAttribute
     * @return
     */
    public static Object getAttribute ( PageContext pageContext, String sAttribute )
    {
        //--- Try to find the "current bean" object 
        Object oBean = Page.getCurrentBean(pageContext);
	    if ( oBean != null  &&  sAttribute != null )
	    {
	        try
	        {
	            return BeanUtil.getFieldValue(oBean, sAttribute);
	        } catch (TelosysException e)
	        {
	            // Unknown field
	            return null ;
	        }                    
	    }
	    return null ;        
    }
}