package org.objectweb.telosys.uil.taglib.widget;

import java.util.Iterator;

import org.objectweb.telosys.uil.taglib.Finder;
import org.objectweb.telosys.uil.taglib.LoopManager;
import org.objectweb.telosys.uil.taglib.Page;
import org.objectweb.telosys.uil.taglib.widget.html.ListBodyHTML;


/**
 * @author Laurent GUERIN
 *  
 */
public class ListBody extends GenericTag
{
    protected static final String TAG_NAME = "listbody";

    private static final int NO_TYPE = 0 ;
    
    private static final int LOOP_TYPE = 1 ;
    
    private static final int DATA_TYPE = 2 ;
    
    //----------------------------------------------------------------
    // TAG generators ( thread safe => static )
    //----------------------------------------------------------------
    private final static IWidget  $htmlGen = new ListBodyHTML(); 

    private final static IWidget  $xulGen  = null;

    //----------------------------------------------------------------
    // TAG attributes
    //----------------------------------------------------------------
    
    //private String              _sProvider = null ; // Replaced by "data" + "scope"

    private String              _sScope    = null ;
    
    private String              _sLoop     = null ;

    //----------------------------------------------------------------
    // Private variables
    //----------------------------------------------------------------
//    private int    _iCount = 0;
//
//    private int    _iIndex = 0;
//
//    private int    _iFrom  = 0;
//
//    private int    _iTo    = 0;

    //private int    _iType    = NO_TYPE;
    
    //private LoopManager _loop = null ;
    
    /**
     * Constructor
     */
    public ListBody()
    {
        super(TAG_NAME, $htmlGen, $xulGen);
    }

    //----------------------------------------------------------------
    public void setProvider(String v) // Deprecated (same as "data" attribute )
    {
        //_sProvider = v;
        setData(v);
    }

//    public String getProvider()
//    {
//        return _sProvider ;
//    }

    //----------------------------------------------------------------
    public void setScope(String v)
    {
        _sScope = v;
    }
    public String getScope()
    {
        return _sScope ;
    }

    //----------------------------------------------------------------
    public void setLoop(String v)
    {
        _sLoop = v;
    }

    public String getLoop()
    {
        return _sLoop ;
    }

    //----------------------------------------------------------------
    private int getIterationType()  
    {
        int iType = NO_TYPE ;
        if ( _sLoop != null )
        {
            iType = LOOP_TYPE ;
        }
        //if ( _sProvider != null ) // override LOOP_TYPE
        if ( getData() != null ) // override LOOP_TYPE
        {
            iType = DATA_TYPE ;
        }        
        return iType;
    }
    //----------------------------------------------------------------
//    private boolean init()  
//    {
//        _iType = NO_TYPE ;
//        if ( _sLoop != null )
//        {
//            parseLoop(_sLoop);
//            _iType = LOOP_TYPE ;
//        }
//        if ( _sProvider != null ) // override LOOP_TYPE
//        {
//            _iType = DATA_PROVIDER_TYPE ;
//        }
//        
//        return true;
//    }
    
    //----------------------------------------------------------------
    //----------------------------------------------------------------
    private LoopManager initLoopManager()
    {
        //String sProviderToUse = _sProvider ;
        String sListDataToUse = getData() ;
        int iType = getIterationType();
        if ( iType != LOOP_TYPE && iType != DATA_TYPE)
        {
            //--- Type not defined no "loop" and no "provider"
            // By default, try to find a RENDERER-BEAN as provider
            sListDataToUse = "RENDERER-BEAN" ;
            iType = DATA_TYPE ;
        }
        
        LoopManager loopManager = null ;

        if ( iType == LOOP_TYPE ) // attribute loop="" 
        {
            //--- Init a loop manager 
            loopManager = new LoopManager ( pageContext, _sLoop );   
            //System.out.println(" initLoopManager :  " + loopManager );
        }
        else if ( iType == DATA_TYPE ) // attribute provider="" 
        {
//            try
//            {
//                Iterator iter = Finder.findIterator(pageContext, _sProvider, null);
//                //--- Init a loop manager ( with the ITERATOR )
//                if ( iter != null )
//                {
//                    loopManager = new LoopManager ( pageContext, iter );   
//                    System.out.println(" initLoopManager :  " + loopManager );                    
//                }
//            } catch (TelosysException e)
//            {
//                // TODO Auto-generated catch block
//                e.printStackTrace();
//            } 
            Iterator iter = Finder.findIterator(pageContext, sListDataToUse, null);
            //--- Init a loop manager ( with the ITERATOR )
            if ( iter != null )
            {
                loopManager = new LoopManager ( pageContext, iter );   
                //System.out.println(" initLoopManager :  " + loopManager );                    
            }
        }
        
        return loopManager ;
    }
    //----------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.jsp.tagext.Tag#doStartTag()
     */
    public int doStartTag()
    {
        startTag();
        LoopManager loopManager = initLoopManager();
        
        //--- Set the loop manager in the current page context ( if NULL => removed )
        Page.setLoopManager(pageContext, loopManager);
        
        //--- If there's a FIRST ITEM in the loop
        if ( loopManager != null )
        {
            if ( loopManager.first() )
            {
                return EVAL_BODY_INCLUDE;
            }
        }
        return SKIP_BODY;        
    }
    //----------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.jsp.tagext.IterationTag#doAfterBody()
     */
    public int doAfterBody()
    {
        //--- Retrieve the LOOP MANAGER
        LoopManager loopManager = Page.getLoopManager(pageContext) ;   
        //--- If there's a NEXT ITEM in the loop
        if ( loopManager != null )
        {
            if ( loopManager.next() )
            {
                //--- Another iteration ...
                //System.out.println(" doAfterBody : EVAL_BODY_AGAIN ");
                return EVAL_BODY_AGAIN;
            }
        }

        //--- It's the end ( no other iteration ) 
//        System.out.println(" doAfterBody : SKIP_BODY ");
        return SKIP_BODY;                                        
    }

    //----------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag()
    {
//        System.out.println(" doEndTag ");
        //--- It's the end ( no other iteration ) 
        endTag();
        return EVAL_PAGE; // EVAL_PAGE = continue , SKIP_PAGE = abort the processing                                      
    }
}
