package org.objectweb.telosys.uil.taglib.widget.html;

import java.io.IOException;

import javax.servlet.jsp.JspWriter;

import org.objectweb.telosys.uil.taglib.TagCommons;
import org.objectweb.telosys.uil.taglib.widget.FileUpload;
import org.objectweb.telosys.uil.taglib.widget.GenericTag;
import org.objectweb.telosys.uil.taglib.widget.IWidget;

/**
 * HTML generation for the widget
 * 
 * @author Laurent GUERIN
 *  
 */
public class FileUploadHTML extends GenericHTML implements IWidget 
{

    //----------------------------------------------------------------

    public void startTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        FileUpload tag = (FileUpload) genericTag ;
        // <form ...>
        generateFormTag(out, tag);
        // <input ...>
        generateInputTag(out, tag);
        // </form>
        out.println("</form>");
        // <iframe ...>
        generateIframeTag(out, tag);
    }
    
    //----------------------------------------------------------------
    public void endTag(JspWriter out, GenericTag genericTag) throws IOException
    {
    }
    
    //----------------------------------------------------------------
    private String addParam( String sName, String sValue, int i ) //throws IOException 
    {
        if ( i < 1 )
        {
            return "?" + sName + "=" + sValue.trim() ;
        }
        else
        {
            return "&" + sName + "=" + sValue.trim() ;            
        }        
    }
    
    //----------------------------------------------------------------
    private String buildActionURL( FileUpload tag ) //throws IOException 
    {
        String sActionURL = TagCommons.getUploadURL( tag.getPageContext() );
        int i = 0 ;
    	if (notVoid(tag.getMaxsize())) 
    	{
    	    sActionURL += addParam( "max_size", tag.getMaxsize(), i++ ) ;
    	}
    	if (notVoid(tag.getDestdir())) 
    	{
    	    sActionURL += addParam( "dest_dir", tag.getDestdir(), i++ ) ;
    	}
    	if (notVoid(tag.getContextid())) {
    	    sActionURL += addParam( "context_id", tag.getContextid(), i++ ) ;
    	}
    	if (notVoid(tag.getContextattribute())) 
    	{
    	    sActionURL += addParam( "context_attribute", tag.getContextattribute(), i++ ) ;
    	}
    	if (notVoid(tag.getSessionkey())) 
    	{
    	    sActionURL += addParam( "session_key", tag.getSessionkey(), i++ ) ;
    	}
    	return sActionURL ;
    }
    
    private void generateFormTag(JspWriter out, FileUpload tag) throws IOException 
    {
    	// Form id method enctype target
//    	out.print("<form id=\"" + tag.getId() + "_form\" " );
//    	out.print(tag.getId());
//    	out.print("_form\" method=\"post\" enctype=\"multipart/form-data\" target=\"" );
//    	out.print(tag.getId());
//    	out.print("_iframe\" action=\"file.upload?");
    	
        String sActionURL = buildActionURL(tag) ;
        
    	out.print("<form" );
    	out.print(" id=\"" + tag.getId() + "_form\"" );
    	out.print(" method=\"post\"" );
    	out.print(" enctype=\"multipart/form-data\"" );
    	out.print(" target=\"" + tag.getId() + "_iframe\"" );
    	out.print(" action=\"" + sActionURL + "\"" );
    	out.println(" >");
    	/***
    	// Param
    	if (notVoid(tag.getMaxsize())) {
    		out.print("max_size=");
    		out.print(tag.getMaxsize());
    		out.print("&");
    	}
    	if (notVoid(tag.getDestdir())) {
    		out.print("dest_dir=");
    		out.print(tag.getDestdir());
    		out.print("&");
    	}
    	if (notVoid(tag.getContextid())) {
    		out.print("context_id=");
    		out.print(tag.getContextid());
    		out.print("&");
    	}
    	if (notVoid(tag.getContextattribute())) {
    		out.print("context_attribute=");
    		out.print(tag.getContextattribute());
    		out.print("&");
    	}
    	if (notVoid(tag.getSessionkey())) {
    		out.print("session_key=");
    		out.print(tag.getSessionkey());
    	}
    	// End tag
    	out.println("\" >");
    	**/
    }
    
    //  ----------------------------------------------------------------
	private void generateInputTag(JspWriter out, FileUpload tag) throws IOException 
	{
		out.print("<input type=\"file\" name=\"file\" id=\"");	
		out.print(tag.getId());
		out.print("\" ");
		if (notVoid(tag.getWidth())){
			int width = Integer.parseInt(tag.getWidth());
			width /= 6;
			out.print("size=\"");
			out.print(width);
			out.print("\" ");
		}
		out.print("style=\"");
		out.print(getStyleFragPosition(tag));
		out.print(getStyleFragWidthHeight(tag));
		out.println("\" > </input>");
	}

    //  ----------------------------------------------------------------
	private void generateIframeTag(JspWriter out, FileUpload tag) throws IOException {
		out.print("<iframe id=\"");
		out.print(tag.getId());
		out.print("_iframe\" name=\"");
		out.print(tag.getId());
		out.print("_iframe\" src=\"about:blank\" onload=\"fwkUploadCallBack(this");
		if (notVoid(tag.getCallback())) {
			out.print(",");
			out.print(tag.getCallback());
		}
		out.println(")\" style=\"width:0px;height:0px;display:none;\" ></iframe>");
	}

}